//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
//  Copyright Syncfusion Inc. 2001 - 2003. All rights reserved. Use of this code is subject to the terms of our 
//  license. A copy of the current license can be obtained at any time by e-mailing licensing@syncfusion.com. 
//  Re-distribution in any form is strictly prohibited. Any infringement will be prosecuted under applicable laws. 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
 


namespace GridGridinCell
{
	using System;
	using System.Drawing;
	using System.Collections;
	using System.ComponentModel;
	using System.Windows.Forms;
	using System.Data;
	using System.Runtime.Serialization;

	using Syncfusion.Diagnostics;
	using Syncfusion.Windows.Forms;
	using Syncfusion.Windows.Forms.Grid;

	/// <summary>
	/// Summary description for GridInCell.
	/// </summary>
	public class GridInCellModel : GridGenericControlCellModel
		{
			protected GridInCellModel(SerializationInfo info, StreamingContext context)	: base(info, context)
			{
			}
			public GridInCellModel(GridModel grid): base(grid)
			{
				AllowFloating = false;
			}
			public override GridCellRendererBase CreateRenderer(GridControlBase control)
			{
				return new GridInCellRenderer(control, this);
			}
		}
	
		public class GridInCellRenderer: GridGenericControlCellRenderer
		{
			private CellEmbeddedGrid activeGrid;
			public GridInCellRenderer(GridControlBase grid, GridCellModelBase cellModel): base(grid, cellModel)
			{
				this.SupportsFocusControl = true;
			}

			protected override void OnDraw(System.Drawing.Graphics g, System.Drawing.Rectangle clientRectangle, int rowIndex, int colIndex, Syncfusion.Windows.Forms.Grid.GridStyleInfo style)
			{
				if (this.ShouldDrawFocused(rowIndex, colIndex))
				{
					if (style.Control is CellEmbeddedGrid)
						activeGrid = (CellEmbeddedGrid) style.Control;
					base.OnDraw(g, clientRectangle, rowIndex, colIndex, style);
				}
				else
				{
					// Draw a static grid
					if (style.Control is CellEmbeddedGrid)
					{
						CellEmbeddedGrid grid = (CellEmbeddedGrid) style.Control;
						grid.DrawGrid(g, clientRectangle, true);
					}
				}
			}

			protected override bool ProcessKeyEventArgs(ref Message m)  
			{
				TraceUtil.TraceCurrentMethodInfo(m.ToString());

				// forward keyboard events to child grid that would otherwise 
				// be handled by parent grid (right arrow, page down etc.)
				if (activeGrid != null && activeGrid.Focused)
					return activeGrid.InitiateProcessKeyEventArgs(ref m);
				return base.ProcessKeyEventArgs(ref m);
			}

			protected override void OnClick(int rowIndex, int colIndex, MouseEventArgs e)
			{
				//only call baseclass if no current cell
//				if(!this.activeGrid.CurrentCell.HasCurrentCell)
//					base.OnClick(rowIndex, colIndex, e);
				if(this.activeGrid == null || !this.activeGrid.CurrentCell.HasCurrentCell)
					base.OnClick(rowIndex, colIndex, e);

			}
		}

		public class CellEmbeddedGrid : GridControl
		{
			public delegate void SetParentHandler(GridControl parent,GridControl grid);
			private GridCurrentCell cell;
			public CellEmbeddedGrid(GridControl parent)
			{
				this.RowHeights[0] = this.RowHeights[1];
				this.DefaultColWidth = 40;
			
				//this.TransparentBackground = true;
				this.SupportsTransparentBackColor = true;
				this.BorderStyle = BorderStyle.FixedSingle;
				this.FloatCellsMode = GridFloatCellsMode.OnDemandCalculation;
				this.VScrollBehavior = GridScrollbarMode.Disabled;
				this.HScrollBehavior = GridScrollbarMode.Disabled;
				this.BorderStyle = BorderStyle.FixedSingle;
				this.Location = new Point(-10000, -10000);
				this.ActivateCurrentCellBehavior = GridCellActivateAction.ClickOnCell;
				this.TableStyle.CellType = "Static";
				this.ShowCurrentCellBorderBehavior = GridShowCurrentCellBorder.GrayWhenLostFocus;
				this.VerticalThumbTrack = true;
				this.HorizontalThumbTrack = true;
				this.FillSplitterPane = false;
				this.DefaultGridBorderStyle = GridBorderStyle.Dotted;
				this.Properties.RowHeaders = false;
				this.ColCount = 0;
				this.RowCount = 0;
				this.TableStyle.WrapText = false;
				this.BaseStylesMap["Standard"].StyleInfo.Clickable = true;
				this.TableStyle.Clickable = true;
				this.BaseStylesMap["Standard"].StyleInfo.WrapText = false;
				this.BaseStylesMap["Header"].StyleInfo.WrapText = false;
				this.BaseStylesMap["Column Header"].StyleInfo.WrapText = false;
				this.AllowSelection = (GridSelectionFlags.AlphaBlend | GridSelectionFlags.Row | GridSelectionFlags.Cell);
				this.ListBoxSelectionMode = SelectionMode.One;
				//			if(parent.InvokeRequired)
				//				parent.Invoke(new SetParentHandler(SetParent),new object[] {parent,this});
				//			else
				//				SetParent(parent,this);
				this.Properties.BackgroundColor = Color.White;

				this.LostFocus += new System.EventHandler(Grid_LostFocus);
				this.GotFocus += new System.EventHandler(Grid_GotFocus);
			}
		
			private void Grid_LostFocus(object sender,EventArgs e)
			{
				this.Selections.Clear();
				this.Refresh();
			}

			private void Grid_GotFocus(object sender,EventArgs e)
			{
				this.Selections.Add(GridRangeInfo.Cells(this.CurrentCell.RowIndex,0,this.CurrentCell.RowIndex,this.ColCount));
				this.Refresh();
			}
			private void SetParent(GridControl parent,GridControl grid)
			{
				grid.Parent = parent;
			}

			protected override void OnKeyDown(KeyEventArgs e)
			{
				TraceUtil.TraceCurrentMethodInfo(e.KeyCode);
				base.OnKeyDown(e);
			}

			internal bool InitiateProcessKeyEventArgs(ref Message m)  
			{
				return base.ProcessKeyEventArgs(ref m);
			}
		}
}
